package ch.psi.daq.test.queryrest.query;

import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.LongStream;
import java.util.stream.Stream;

import com.google.common.collect.Lists;

import ch.psi.daq.common.ordering.Ordering;
import ch.psi.daq.domain.DataEvent;
import ch.psi.daq.domain.cassandra.ChannelEvent;
import ch.psi.daq.domain.reader.DataReader;

public class DummyDataReader implements DataReader {

   public static final String TEST_CHANNEL_1 = "testChannel1";
   public static final String TEST_CHANNEL_2 = "testChannel2";
   public static final List<String> TEST_CHANNEL_NAMES = Lists.newArrayList(TEST_CHANNEL_1, TEST_CHANNEL_2);

   @Override
   public Stream<String> getChannelStream(String regex) {
      Stream<String> channelStream = TEST_CHANNEL_NAMES.stream();

      if (regex != null) {
         Pattern pattern = Pattern.compile(regex);
         channelStream = channelStream.filter(channel -> pattern.matcher(channel).find());
      }

      return channelStream;
   }

   @Override
   public Stream<? extends DataEvent> getEventStream(String channel, long startPulseId, long endPulseId,
         Ordering ordering, String... columns) {
      return getElements(channel, startPulseId, endPulseId);
   }

   @Override
   public Stream<? extends DataEvent> getEventStream(String channel, long startMillis, long startNanos, long endMillis,
         long endNanos, Ordering ordering, String... columns) {
      return getElements(channel, startMillis, endMillis);
   }

   protected Stream<? extends DataEvent> getElements(String channel, long start, long end) {
      return LongStream.rangeClosed(start, end).mapToObj(i -> {
         return new ChannelEvent(
               channel,
               i,
               0,
               i,
               i,
               0,
               123 // dummy value
               );
      });
   }
}
